-- author: rafftnix
-- date: 19.11.2016

-- unlocks comp joint rotation limit

UnlockCompJoint = {};

function UnlockCompJoint.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Cylindered, specializations);
end;

function UnlockCompJoint:load(savegame)
	self.setIsCompJointUnlocked = SpecializationUtil.callSpecializationsFunction("setIsCompJointUnlocked");
	
	self.compJointUnlocked = false;
	
	self.compJointsToUnlock = {}
	local num = 0;
	while true do
		local key = "vehicle.unlockCompJoint.compJoint("..tostring(num)..")";
		if not hasXMLProperty(self.xmlFile, key) then
			break;
		end;
		
		local compJoint = {}
		compJoint.jointIndex = getXMLInt(self.xmlFile, key.."#index");
		compJoint.movingToolIndex = getXMLInt(self.xmlFile, key.."#movingToolIndex");
		compJoint.directionReference = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.. "#directionReference"));
		compJoint.directionFrame = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.. "#directionFrame"));
		compJoint.middleRot = Utils.getRadiansFromString(getXMLString(self.xmlFile, key.. "#middleRot"), 3);
		compJoint.movingToolRotSpeedBackup = self.movingTools[compJoint.movingToolIndex].rotSpeed;
		table.insert(self.compJointsToUnlock, compJoint);
		num = num + 1;
	end;
end;

function UnlockCompJoint:delete()
  
end;

function UnlockCompJoint:readStream(streamId, connection)
    local isUnlocked = streamReadBool(streamId);
	self:setIsCompJointUnlocked(isUnlocked, true);
end;

function UnlockCompJoint:writeStream(streamId, connection)
    streamWriteBool(streamId, self.compJointUnlocked);
end;

function UnlockCompJoint:mouseEvent(posX, posY, isDown, isUp, button)
end;

function UnlockCompJoint:keyEvent(unicode, sym, modifier, isDown)
end;

function UnlockCompJoint:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			self:setIsCompJointUnlocked(not self.compJointUnlocked);
		end;
	end;
end;

function UnlockCompJoint:updateTick(dt)
end;

function UnlockCompJoint:draw()
	if not self.compJointUnlocked then
		g_currentMission:addHelpButtonText(g_i18n:getText("action_unlockCompJoint"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("action_lockCompJoint"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
	end;
end;

function UnlockCompJoint:setIsCompJointUnlocked(isUnlocked, noEventSend)
	self.compJointUnlocked = isUnlocked;
	
	if self.isServer then
		for a=1, table.getn(self.compJointsToUnlock) do
			local compJoint = self.compJointsToUnlock[a];
			local movingTool = self.movingTools[compJoint.movingToolIndex];
			local maxJointRot = 0;
			
			if isUnlocked then
				movingTool.rotSpeed = 0;
				maxJointRot = (movingTool.rotMax - movingTool.rotMin); 
				setRotation(self.componentJoints[compJoint.jointIndex+1].jointNode, unpack(compJoint.middleRot));
				maxJointRot = (movingTool.rotMax - movingTool.rotMin)/2; 
			else
				movingTool.rotSpeed = compJoint.movingToolRotSpeedBackup;
				local x, y, z = getRotation(movingTool.node);
				movingTool.curRot = {x, y, z};
				
				local x, y, z = getWorldTranslation(compJoint.directionReference);
				local dirX, dirY, dirZ = worldToLocal(movingTool.node, x, y, z);
				local dirX, dirY, dirZ = localDirectionToWorld(movingTool.node, dirX, dirY, dirZ);
				local upX, upY, upZ;
				if compJoint.directionFrame == nil then 
					upX, upY, upZ = localDirectionToWorld(movingTool.node, 0, 1, 0);
				else
					upX, upY, upZ = localDirectionToWorld(compJoint.directionFrame, 0, 0, 1);
				end;
				Utils.setWorldDirection(movingTool.node, dirX, dirY, dirZ, upX, upY, upZ, nil, nil, nil);
				
				Cylindered.updateComponentJoints(self, movingTool, false);
			end;
			
			setJointRotationLimit(self.componentJoints[compJoint.jointIndex+1].jointIndex, movingTool.rotationAxis-1, true, -maxJointRot, maxJointRot);
		end;
	end;
	
	if not notEventSend then
		UnlockCompJointEvent.sendEvent(self, isUnlocked, noEventSend);
	end;
end;